% LICENSE AGREEMENT
% - - - - - - - - -
% Copyright (c) 2019, Stilianos Louca
% All rights reserved.
%
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are
% met:
%
%    * Redistributions of source code must retain the above copyright
%      notice, this list of conditions and the following disclaimer.
%    * Redistributions in binary form must reproduce the above copyright
%      notice, this list of conditions and the following disclaimer in
%      the documentation and/or other materials provided with the distribution
%    * Neither the name of the original author (Stilianos Louca), nor the names
%      of any other contributors may be used to endorse or promote products derived
%      from this software without specific prior written permission.
%
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
% AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
% IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
% ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
% LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
% CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
% SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
% CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
% ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
% POSSIBILITY OF SUCH DAMAGE.
%
% - - - - - - - - -
% Code name and version: SMF_example_redox_shuttles 2019.02.07
% Tested on MATLAB R2015b.
% - - - - - - - - -
%
% DESCRIPTION
% - - - - - -
% This MATLAB code demonstrates Spatial Metabolic Flux analysis in a marine sediment column, under the following scenario: 
%   oxMnII: Dissolved MnII is oxidized to particulate MnO2 using oxygen (O2) diffusing from the top.
%   rMnO2: Precipitating particulate MnO2 (produced during MnII oxidation) is reduced usign hydrogen sulfide (H2S) diffusing from the bottom.
%   No other reaction is assumed to occur.
%   The above reactions (oxMnII and rMnO2) take place at two separate hotspots.
%   Metabolite concentrations (O2, H2S, MnII, MnO2) at the top and bottom boundaries of the system are assumed to be fixed.
%	All metabolites except MnO2 are assumed to be transported via eddy diffusion; MnO2 is assumed to be transported purely advectively (i.e., by sinking, with diffusion being negligible)
% The resulting steady-state metabolite concentrations profiles across depth, the vertical metabolite fluxes and the reaction rates are all predicted via SMF.
% 
% This scenario is a classical example of a metal redox shuttle, mediating the indirect oxidation of H2S with O2 using Mn as intermediates.
% Literature documenting this scenario:
%	Percy et al. (2008). Controls on iron, manganese and intermediate oxidation state sulfur compounds in the Cariaco Basin. Marine Chemistry. 111:47-62
%	Bacon et al. (1980). Lead-210, polonium-210, manganese and iron in the Cariaco Trench. Deep Sea Research A. 27:119-135
%	Boudreau (1991). Modelling the sulfide-oxygen reaction and associated pH gradient in porewaters. Geochimica et Cosmochimica Acta. 55:145-159.
%
% The code performs the following main tasks:
%   1. Specifies the model (including metabolites, reactions, stoichiometric matrix and diffusivity) for the SMF analysis
%   2. Runs the SMF analysis, predicting the hotspot locations and reaction rates. 
%   3. Plots the predicted H2S, O2, MnII and MnO2 depth profiles and prints out the predicted reaction rates.
% Task 2 is performed by the accompanied SMF workflow in file "fitSMFmodel.m".
% Task 1 is performed in this source file, while task 3 is performed using auxiliary plotting routines provided in separate source files ("plotProfiles.m" and "num2RowsColumns.m").
% Typically, you would use your own code to perform tasks 1 and 3.
%
% Time is in days, depth in meters (m), metabolite concentrations in mol/litre (M), diffusivity in m^2/day, reaction rates in kmol/m^2/day.
%
% Tested on MATLAB R2015b.
% To run this code, simply run 'main' in the MATLAB console.
% 
% Stilianos Louca
% February 12, 2019
%
function main()

	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	% TECHNICAL PREPARATIONS & OPTIONS
	
	% figure out uniquely numbered output dir
	outputDir = getNonExistingSimilarDir('output', 'example_run_');
	makedir(outputDir);
			
	% output precision
	format long;
		
	fprintf('Initializing..\nNote: All outputs and plots will be saved in the directory "%s"\n', outputDir);
	
	% make backup copies of code
	makedir(strcat(outputDir,'/code_backup'));
	sources = dir('*.m');
	for s=1:length(sources)
		copyfile(sources(s).name, strcat(outputDir, '/code_backup/', sources(s).name)); 
	end
	
	PLOT_WIDTH	= 3;	% depth-profile plot width, in inches
	PLOT_HEIGHT	= 3;	% depth-profile plot height, in inches

	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	% SPECIFY MODEL FOR SMF ANALYSIS
	
	% define considered metabolites & reactions
	% Substances not limiting to any reaction (such as CO2, H2O, SO4 or H+) can be ignored - including them wouldn't change the SMF predictions
	% Element- and electron-balancing by each reaction is not required for SMF analysis
	reaction_names 	 = {'oxMnII' 'rMnO2'};
	metabolite_names = {'H2S' 'O2' 'MnII' 'MnO2'};
	
	% define reaction stoichiometry
	% This 2D matrix must be consistent with the metabolites & reactions defined above
	% The entry in row m and column r is the stoichiometric coefficient of metabolite m in reaction r
	%					oxMnII	rMnO2
	stoichiometry =	[ 	 0		-1;		% H2S
						-1		 0;		% O2
						-2		+4;		% MnII
						+2		-4];	% MnO2


	% check if dimensions are correct
	NR = numel(reaction_names);
	NM = numel(metabolite_names);
	if((size(stoichiometry,1)~=NM) || (size(stoichiometry,2)~=NR))
		fprintf('ERROR: Stoichiometric matrix must have %d rows and %d columns, but instead has %d rows and %d columns\n', NM, NR, size(stoichiometry,1), size(stoichiometry,2));
		return;
	end
	
	% specify boundary concentrations (in mol/L) at top & bottom of the water column
	% at each boundary (top & bottom), boundary concentrations should be 1D vectors of size NM
	% top boundary = water-sediment interface
	% bottom boundary = arbitrary point in sediment column below the sulfate-methane transition zone, above which total methane production is negligible.
	concentrations_top = [	0;			% H2S
							60e-6;	 	% O2 % typical O2 concentrations in Cariaco Basin at depth 150 m
							0;			% MnII
							0];			% MnO2
	concentrations_bottom = [	20e-6;	% H2S % typical H2S concentration in Cariaco Basin at depth 500 m
								0;	 	% O2
								10e-6; %  MnII % based on MnII concentrations reported for Cariaco Basin at depth 500 m [Bacon et al. (1980). Lead-210, polonium-210, manganese and iron in the Cariaco Trench. Deep Sea Research A. 27:119-135. Fig. 6]
								NaN];	% MnO2 % for MnO2, we only need one boundary condition since transport is assumed to be purely advective, so leave the boundary condition at the bottom unspecified
			
	% specify the modeled number of hotspots
	% hotspots are identified in order of increasing depth
	NH = 2;
	
	% constrain some of the possible reaction rates in the model (e.g. based on typical empirical observations).
	% Constraining reaction rates helps the model converge on a more plausible prediction; however leaving all reactions unconstrained will typically also lead to a plausible prediction.
	% Constraints must be specified as a 2D numeric array of size NR x NH.
	% Entry at row r and column h specifies the rate of reaction r at hotspot h. NaN indicates an unconstrained reaction rate.
	fixed_reaction_rates = [NaN		0; 		% oxMnII. Expect no oxMnII activity in the lower hotspot
							0		NaN]; 	% rMnO2. Expect no rMnO2 activity in the upper hotspot
	
	% specify the depth-grid to be considered, including the top and bottom boundary
	% a finer grid increases accuracy, but also increases computation time
	SMF_DEPTHS = linspace(150, 500, 200);
	NZ = numel(SMF_DEPTHS);

	% Specify diffusivity for each metabolite (in m^2/day)
	% Assume eddy (turbulent) diffusivity is the main mode of transport for almost all metabolites, except for MnO2 whose transport is assumed to be purely advective (sinking)
	eddy_diffusivity = @(z) 67; % Time-averaged eddy diffusivity estimated for Cariaco Basin at depth 300 (i.e. within the redoxcline) during years 2001-2014 [Louca et al. (in review as of 2019). Microbial metabolite fluxes in a model marine anoxic ecosystem]
	MnO2_diffusivity = @(z) zeros(size(z)); % assume zero diffusivity for MnO2, use eddy diffusivity for all other metabolites
	diffusivities 	 = {eddy_diffusivity eddy_diffusivity eddy_diffusivity MnO2_diffusivity};
	
	% specify the vertical advection speed for each metabolite (in m/day)
	% Here we assume that advection is zero for all metabolites except for MnO2
	% Yakushev et al. (2013, pp. 11) assumes a MnO2 sinking speed of Vmax*[MnO2]/([MnO2] + 100 nM), where Vmax = 16 m/day. [Percy et al 2008] reports particulate Mn concentrations ~10-50 nM in the Cariaco redoxcline, implying sinking speeds 1.5-5.3 m/day.
	advections = {0 0 0 5};
	
	% plot diffusivity profile
	fprintf('Plotting assumed diffusivity profiles..\n');
	plotProfiles(	diffusivities,...
					SMF_DEPTHS, ...
					{}, ...
					NaN, ...
					[SMF_DEPTHS(1),SMF_DEPTHS(end)], ...
					metabolite_names, ...
					'diffusivity $(m^2/day)$', ...
					strcat(outputDir,'/diffusivities.pdf'),...
					'Metabolite-specific diffusivities assumed in the model', ...	% supertitle
					0,... 				% min x value
					NaN, ... 			% refValue
					false, ...			% logarithmic
					false, ...			% show_points
					PLOT_WIDTH, ...
					PLOT_HEIGHT, ...
					false, ... 	% showDepthIntegrated
					false, ...	% showExtrema
					true, ... 	% showR2P2
					NaN, ...	% data_colors
					strcat(outputDir,'/plot_data/diffusivities.tsv'));


	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	% Run SMF analysis
	
	fprintf('Running SMF between depths %g-%g m. This may take a few minutes..\n',SMF_DEPTHS(1),SMF_DEPTHS(end));
		
	[hotspots,...
	reaction_rates,...
	~,...				% ghost_source_rates. Irrelevant in this example.
	~,...				% ghost_sink_rates. Irrelevant in this example.
	concentrations,...
	concentrations_BC_part,...
	~,...				$ concentrations_added_production_part. Irrelevant in this example.
	concentrations_reaction_part,...
	metabolite_net_source_rates,...
	metabolite_boundary_influxes,...
	stress,...
	~,... 				% R2s_grid_data. Irrelevant in this example.
	~,... 				% R2s_scatter_data. Irrelevant in this example.
	hotspots_all_trials,...
	reaction_rates_all_trials,...
	stress_all_trials,...
	Niterations_all_trials,...
	Nevaluations_all_trials,...
	~,... 				% R2s_grid_data_all_trials. Irrelevant in this example.
	~,... 				% R2s_scatter_data_all_trials. Irrelevant in this example.
	~,...				% spurious_hotspots. Irrelevant in this example.
	~,...				% neglibible_hotspots. Irrelevant in this example.
	exit_flag,...
	Niterations,...
	Nfailed_trials,...
	errormsg] = fitSMFmodel(SMF_DEPTHS,...				% gridZ,...
							NM,... 						% number of metabolites
							NR,...						% number of reactions
							NH,...						% number of hotspots
							metabolite_names,...
							reaction_names,...
							zeros(NM,1),...				% BCtypesA. All boundary conditions are Dirichlet type (fixed concentration).
							zeros(NM,1),...				% BCtypesB. All boundary conditions are Dirichlet type (fixed concentration).
							concentrations_top,...		% BCvaluesA
							concentrations_bottom,...	% BCvaluesB
							diffusivities,...			% vertical diffusivities
							advections,...				% vertical advection speeds
							@(z) 1,...					% dilution_area (as function of depth). Lateral (cross-sectional) basin area is assumed to be constant in this example, i.e. assume no geometric dilution effects.
							NaN,...						% grid_temperatures. Irrelevant in this example.
							true,...					% use_ficks_law
							stoichiometry,...			% reaction_stoichiometry
							NaN,...						% reaction_DeltaG0. Irrelevant in this example.
							{},...						% concentration_grid_data
							{},...						% concentration_scatter_data
							[],...						% added_production_rates,...
							[],...						% ghost_sources (optional)
							[],...						% ghost_sinks (optional)
							1,...						% metabolite_weights. All metabolites weighted equally in this example.
							1e-6,...					% positivity_scales. Reference scales for metabolites for penalizing positive concentrations due to inefficient resource use.
							1e-6,...					% negativity_scales. Irrelevant in this example, since positivity of concentrations is strictly enforced (negative_concentration_penalty=Inf).
							'min_substrate_concentrations',...	% how to penalize inefficiencies of substrate use (i.e. deviations from SMF assumption 4). Recommended options are either 'min_substrate_concentrations' or 'substrate_geomean'. Which one works best sometimes depends on the particular model.
							Inf,...						% negative_concentration_penalty
							0,...						% negative_flux_penalty
							false,...					% include_grid_data
							false,...					% include_scatter_data
							[],...						% start_hotspots (optional)
							[],...						% start_reaction_rates (optional)
							[],...						% always_use_start_hotspots
							true,...					% always_use_start_reaction_rates
							[],...						% fixed_hotspots (optional)
							fixed_reaction_rates,...	% fixed_reaction_rates (constrain possible reaction rates predicted by the model)
							[-Inf +Inf],...				% allowed_hotspot_range (optional)
							'fmincon',...				% optim_method
							500,...						% optim_max_iter
							5000*10*NH*NR,...			% optim_max_evals
							1e-7,...					% objective_tolerance
							1e-9,...					% parameter_tolerance
							1e-12,...					% concentration_epsilon
							1000,...					% max_random_rate_factor
							500*NH,...					% Ntrials
							1,...						% Nrounds
							1,...						% Nstart_attempts
							true,...					% parallel
							false,...					% omit_spurious_hotspots
							-1,...						% negligible_rel_flux_threshold
							false,...					% R2s_for_all_trials
							true,...					% check_nans_in_input
							true,...					% verbose
							'  ');						% verbose_prefix

	if(~strcmp(errormsg,''))
		% SMF failed
		fprintf('  ERROR:  SMF failed: %s\n',errormsg);
	else
		% SMF succeeded, so print summary and plot predictions.
		fprintf('Finished SMF:\n  stress: %g\n  Nfailed_trials: %d\n  Niterations: %d\n',stress,Nfailed_trials,Niterations);
		for h=1:NH
			fprintf('Hotspot %d:\n',h);
			fprintf('  Predicted hotspot depth: %g m\n',hotspots(h));
			for r=1:NR
				fprintf('  Predicted %s reaction rate: %g mol/m^2/day\n',reaction_names{r},reaction_rates(r,h));
			end
			for m=1:NM
				fprintf('  Predicted net %s consumption rate: %g mol/m^2/day\n',metabolite_names{m},-metabolite_net_source_rates(m,h));
			end
		end
	
		fprintf('Plotting SMF-predicted metabolite concentrations..\n');
		plotProfiles(	concentrations',...
						SMF_DEPTHS, ...
						{}, ...
						NaN, ...
						[SMF_DEPTHS(1),SMF_DEPTHS(end)], ...
						metabolite_names, ...
						'concentration (mol/L)', ...
						strcat(outputDir,'/predicted_metabolite_concentrations.pdf'),...
						sprintf('SMF-predicted metabolite concentrations (stress=%g)',stress), ...
						0,... 			% min x value
						NaN, ... 		% refValue
						false, ...		% logarithmic
						false, ...		% show_points
						PLOT_WIDTH, ...
						PLOT_HEIGHT, ...
						false, ... 	% showDepthIntegrated
						false, ...	% showExtrema
						true, ... 	% showR2P2
						NaN, ...	% data_colors
						strcat(outputDir,'/plot_data/predicted_metabolite_concentrations.tsv'));
				
		fprintf('Plotting SMF-predicted reaction rates..\n');
		rates_profiles = zeros(NZ,NR);
		for h=1:NH
			[~,grid_point] = min(abs(SMF_DEPTHS-hotspots(h)));
			rates_profiles(grid_point,:) = rates_profiles(grid_point,:) + reaction_rates(:,h)';
		end
		plotProfiles(	rates_profiles,...
						SMF_DEPTHS, ...
						{}, ... 				% data
						NaN, ...
						[SMF_DEPTHS(1),SMF_DEPTHS(end)], ...
						reaction_names, ...
						'reaction rate ($\rm kmol/(m^2\cdot day)$)', ...
						strcat(outputDir,'/predicted_reaction_rates.pdf'),...
						sprintf('SMF-predicted area-specific reaction rates (stress=%g)',stress), ...
						NaN,... 			% min x value
						NaN, ... 			% refValue
						false, ...			% logarithmic
						false, ...			% show_points
						PLOT_WIDTH, ...
						PLOT_HEIGHT, ...
						false, ... 	% showDepthIntegrated
						false, ...	% showExtrema
						false, ... 	% showR2P2
						NaN, ...	% data_colors
						strcat(outputDir,'/plot_data/predicted_reaction_rates.tsv'));		
				
		fprintf('Plotting SMF-predicted metabolite net source rates..\n');
		rates_profiles = zeros(NZ,NM);
		for h=1:NH
			[~,grid_point] = min(abs(SMF_DEPTHS-hotspots(h)));
			rates_profiles(grid_point,:) = rates_profiles(grid_point,:) + metabolite_net_source_rates(:,h)';
		end
		plotProfiles(	rates_profiles,...
						SMF_DEPTHS, ...
						{}, ... 				% data
						NaN, ...
						[SMF_DEPTHS(1),SMF_DEPTHS(end)], ...
						metabolite_names, ...
						'net source rate ($\rm kmol/(m^2\cdot day)$)', ...
						strcat(outputDir,'/predicted_metabolite_net_source_rates.pdf'),...
						sprintf('SMF-predicted area-specific metabolite net source rates (stress=%g)',stress), ...
						NaN,... 			% min x value
						NaN, ... 			% refValue
						false, ...			% logarithmic
						false, ...			% show_points
						PLOT_WIDTH, ...
						PLOT_HEIGHT, ...
						false, ... 	% showDepthIntegrated
						false, ...	% showExtrema
						false, ... 	% showR2P2
						NaN, ...	% data_colors
						strcat(outputDir,'/plot_data/predicted_metabolite_net_source_rates.tsv'));	

		fprintf('Plotting optimization stats for all fitting trials..\n');
		plotScatter(4, ...
					[stress_all_trials stress_all_trials stress_all_trials stress_all_trials],...
					[Niterations_all_trials Nevaluations_all_trials hotspots_all_trials(:,1) hotspots_all_trials(:,2)], ...
					[],...
					NaN, ...
					NaN, ...
					{'Niterations vs stress', 'Nevaluations vs stress' 'hotspot-1 vs stress' 'hotspot-2 vs stress'}, ...
					[], ...
					[], ...
					{'stress', 'stress' 'stress' 'stress'}, ...
					{'Niterations', 'Nevaluations' 'hotspot depth 1 (m)' 'hotspot depth 2 (m)'}, ...
					sprintf('%s/optimization_stats.pdf',outputDir), ...
					sprintf('Optimization stats (stress, Niterations, Nevaluations, hotspot depths) for all SMF fitting trials'), ...
					true, ... 	% xlogarithmic
					false, ...  % ylogarithmic
					false, ... 	% xreverse
					[false false true true], ... 	% yreverse
					PLOT_WIDTH, ...
					PLOT_HEIGHT, ...
					sprintf('%s/plot_data/optimization_stats.tsv',outputDir));

	end
	
	fprintf('Finished. All outputs are in directory "%s"\n',outputDir);
	diary off;
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% AUXILIARY FUNCTIONS


% Get file path to a directory for which there exists no similarly named directory or file
% dir names are chosen incrementally, basename001, basename002 and so on, until no basenameXXX* exists
function filepath = getNonExistingSimilarDir(parent, basename)
	% get all items (level 1) in parent dir
	listing  = dir(sprintf('%s/%s*',parent,basename));
	existing = cell(length(listing),1);
	for l=1:length(listing)
		existing{l} = listing(l).name;
	end
	% increment counter until no similarly named item exists
	n = 1;
	while(true)
		filename = sprintf('%s%03d',basename,n);		
		if(not(any(strncmpi(existing,filename,length(filename)))))
			break;
		end
		n = n+1;
	end
	filepath = sprintf('%s/%s',parent,filename);
end


% create a directory, if it does not exit
function makedir(path)
	if ~exist(path, 'dir')
		mkdir(path);
	end
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


