% LICENSE AGREEMENT
% - - - - - - - - -
% Copyright (c) 2018, Stilianos Louca
% All rights reserved.
%
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are
% met:
%
%    * Redistributions of source code must retain the above copyright
%      notice, this list of conditions and the following disclaimer.
%    * Redistributions in binary form must reproduce the above copyright
%      notice, this list of conditions and the following disclaimer in
%      the documentation and/or other materials provided with the distribution
%    * Neither the name of the original author (Stilianos Louca), nor the names
%      of any other contributors may be used to endorse or promote products derived
%      from this software without specific prior written permission.
%
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
% AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
% IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
% ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
% LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
% CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
% SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
% CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
% ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
% POSSIBILITY OF SUCH DAMAGE.
%
% - - - - - - - - -
% Tested on MATLAB R2015b.
% - - - - - - - - -
%
% DESCRIPTION
% - - - - - -
% Plot depth profiles for N observables at a fixed time
% Profiles can either be given as a 2D matrix (if over the same depth-grid), or as a cell array of profiles (potentially over different depth-grids) or a cell array of function handles (each defining a observable's value over depth).
% data{} is either empty or a cell array of size N: Each element data{m} can be a table of size NDm x 2(one table per observable) or NaN (no data avail).
function plotProfiles(	profiles,...			% 2D array of size NZ x N. profiles(:,j) is depth profile of j-th observable across profileZ[] depths. Alternatively, this can be a cell-array of size N, each entry of which is a matrix of size NZj x 2, listing depths & values (in that case, profileZ is irrelevant) or a function-handle defining the observable's value at each depth.
						profileZ, ...			% 1D vector of size NZ.
						data, ... 
						rescales, ... 			% either NaN or a vector of size N, giving rescaling factors both for data and profiles
						z_range, ...			% either an empty vector, or [min,max]
						titles, ...				% either a cell array of N strings or a single string
						xlabels, ...			% either a cell array of N strings or a single string
						plotFile,...
						supertitle, ...
						minPlotValue,... 		% can be NaN or any real number. Defines the lower x-axis limit
						refValue, ... 			% can be NULL or a real number or a vector of numbers and NaNs. Defines reference value to add to plots (as vertical line)
						logarithmic, ...
						show_points,...			% (boolean) show the individual profile points, in addition to the profile curve
						plotWidth, ...
						plotHeight, ...
						showDepthIntegrated,... % if true, the depth-integrated value for each plot will be shown as a comment in the title
						showExtrema,... 		% if true, the minimum & maximum value for each plot will be shown as a comment in the title
						showR2P2,...			% if true, and if applicable, the coefficient of determination (R2) and the perpendicular coefficient (P2) to the data will be shown as a comment in the title
						data_colors, ... 		% can be NaN or a single string or a cell array of colors for each data set
						dataFile) 				% can be '' (empty)
	if(iscell(profiles))
		N = length(profiles);
	else
		N = size(profiles,2);
	end
	profileZ = reshape(profileZ,[length(profileZ) 1]); % make sure profileZ is column vector
	
	% prepare storage of profiles as cell array
	Zs = cell(N);
	Vs = cell(N);
		
	% geometry for sub-plot distribution
	[nrows ncols] = num2RowsColumns(N, 1.2,0.5,3);
	titleAreaV	= (0.06*2/plotHeight) * (1+length(strfind(supertitle,sprintf('\n')))); % reserve title space
	plotAreaV 	= 1 - titleAreaV;
	
	% initialize figure window (make invisible since we plot everything to pdf)
	fig = figure('Visible','off','PaperPosition',[0 0 ncols*plotWidth nrows*plotHeight],'PaperSize',[ncols*plotWidth nrows*plotHeight]);

	if(size(z_range,1)>0) 
		z_epsilon = (z_range(2) - z_range(1))*1e-3;
	end
	
	% iterate through sub plots
	for row=1:nrows
		for col=1:ncols
			j = (row-1)*ncols+col;
			if(j>N) break; end
			
			hasData = ((length(data)>=j) && (size(data{j},2)>1));
			
			% extract/compute profile for observable j
			if(iscell(profiles))
				if(isa(profiles{j},'function_handle'))
					Z = profileZ;
					V = profiles{j}(profileZ);
					if(numel(V)==1) V = repmat(V,size(Z)); end % vectorize if needed
				elseif(size(profiles{j},2)>1)
					Z = profiles{j}(:,1);
					V = profiles{j}(:,2);
				else
					Z = [];
					V = [];
				end
			else
				Z = profileZ;
				V = profiles(:,j);
			end
			
			% store for latter re-use
			Zs{j} = Z;
			Vs{j} = V;
			
			% omit NaNs
			valids = find(not(isnan(Z)) & not(isnan(V)));
			Z = Z(valids);
			V = V(valids);
			
			% sort data
			[Z, order] = sort(Z);
			V = V(order);
			
			% rescale profiles if necessary
			if(not(isnan(rescales)))
				V = V * rescales(j);
				if(hasData) data{j}(:,2) = data{j}(:,2) * rescales(j); end
			end
			
			% mannually place subplot
			% Note: If subplot overlaps previous subplots, the older ones will be deleted, unless overlap is exact! (this is an idiotic matlab 'feature')
			subplot('Position',[(0.2+col-1)/ncols plotAreaV*(0.25+nrows-row)/nrows 0.65/ncols plotAreaV*0.6/nrows]);
			
			% plot profile
			plot(V,Z,ternary(show_points,'-o','-'),'MarkerSize',0.8*sqrt(plotWidth*plotHeight));
			
			% set x-axis limits
			maxValue = max(V);
			if(hasData)
				if(size(z_range,1)>0) shownData = (data{j}(:,1)<=z_range(end)+z_epsilon) & (data{j}(:,1)>=z_range(1)-z_epsilon); 
				else shownData = 1:size(data{j},1); end
				maxValue = max(maxValue, max(data{j}(shownData,2))); 
			end
			maxValue = ternary(maxValue<=0, 1, 1.2*maxValue);
			if(not(isnan(minPlotValue))) xlim([minPlotValue, maxValue]); end
			if(logarithmic) set(gca, 'XScale', 'log'); end
			
			% set y-axis limits
			if(size(z_range,1)>0) 
				if(hasData)
					ylim([z_range(1)-z_epsilon,z_range(2)+z_epsilon]); % need to 'widen' up plot area a bit to make sure all relevant data points are shown
				else
					ylim(z_range);
				end
			end
			set(gca,'YDir','reverse');
			
			% set title (add various summary stats if needed)
			if(iscell(titles)) thisTitle = titles{j}; else thisTitle = titles; end
			if((length(Z)==0) && (~hasData))
				title(sprintf('%s\nNo data available',sanitize_string_for_latex(thisTitle)));
				continue;
			end
			if(showDepthIntegrated)
				integrated = integrateVector_nansafe(Z, V, false);
				thisTitle = sprintf('%s\ndepth integrated = %.4g', thisTitle, integrated);
			end
			if(showExtrema)
				[min_value, min_i] = nanmin(V);
				[max_value, max_i] = nanmax(V);
				min_z = Z(min_i);
				max_z = Z(max_i);
				thisTitle = sprintf('%s\nmin=%.4g (at depth=%.3g)\nmax=%.4g (at depth=%.3g)', thisTitle, min_value, min_z, max_value, max_z);
			end
			if(showR2P2 & hasData)
				R2 = getProfileR2(Z, V, data{j}(:,1), data{j}(:,2));
				P2 = getPerpendicularR2ToData(Z, V, data{j}(:,1), data{j}(:,2), true);
				thisTitle = sprintf('%s\nR2=%.3g, P2=%.3g', thisTitle, R2, P2);
			end
			title(sanitize_string_for_latex(thisTitle));
			
			ylabel('depth (m)','Interpreter','latex');
			if(iscell(xlabels)) xlabel(sanitize_string_for_latex(xlabels{j}),'Interpreter','latex');
			else xlabel(sanitize_string_for_latex(xlabels),'Interpreter','latex');
			end
			set(gca,'fontsize',8);
			
			% overlay reference value
			if(length(refValue)>1 | not(isnan(refValue)))
				if(length(refValue)==1) rv = refValue;
				elseif(length(refValue)>=j) rv = refValue(j);
				else rv = NaN;
				end
				if(not(isnan(rv)))
					hold on
					plot([rv rv],ylim,'Color',[0.5 0.5 0.5]);
					hold off
				end
			end
			
			% overlay data
			if(hasData)
				if(iscell(data_colors)) color = data_colors{j};
				elseif(isnan(data_colors)) color = 'red';
				else color = data_colors; end
				hold on
				scatter(data{j}(:,2), data{j}(:,1), 6, color);
				hold off
			end
		end
	end
		
	%create super title
	%this should come after subplot(), because the latter has a bad tendency to delete previous figure elements
	annotation('textbox', [0 (1-titleAreaV) 1 titleAreaV], 'String', sanitize_string_for_latex(supertitle), 'EdgeColor', 'none', 'HorizontalAlignment', 'center', 'VerticalAlignment', 'top','Interpreter','latex');

	make_parent(plotFile);
	print(fig,'-dpdf',plotFile);
	
	% Write data to file
	if(not(isempty(dataFile)))
		index = 0;
		make_parent(dataFile);
		fout = fopen(dataFile, 'w');
		fprintf(fout, '# %s\n#\n', strrep(supertitle, sprintf('\n'), sprintf('\n#   ')));
		for j=1:N
			% write j-th profile
			Z = Zs{j}; V = Vs{j};
			if(iscell(titles)) tt = titles{j}; else tt = titles; end
			if(iscell(xlabels)) txl=xlabels{j}; else txl = xlabels; end
			fprintf(fout,'# Data index %d\n# Depth profile: %s\n# Data columns: Depth(m) %s\n', index, tt, txl);
			for zi=1:length(Z)
				fprintf(fout, '%g\t%g\n', Z(zi), V(zi)); 
			end
			fprintf(fout, '\n\n'); index = index + 1;
			% write overlay data (if available)
			if(length(data)>=j)
				if(size(data{j},2)>1)
					fprintf(fout, '# Data index %d\n# Data scatter points: %s\n# Data columns: Depth(m) %s\n', index, tt, txl);
					for di=1:size(data{j},1)
						fprintf(fout, '%g\t%g\n', data{j}(di,1), data{j}(di,2));
					end
					fprintf(fout, '\n\n'); index = index + 1;
				end
			end
		end
		fclose(fout);
	end
end




% Ternary operator for C-like conditional expressions
function c = ternary(condition, a , b)
    if condition
        c = a;
    else
        c = b;
    end
end



% integrate y over its first dimension along x
% x is a vector of length L
% y is a vector of length L or a matrix of size L x M
function I = integrateVector(x, y)
	dx = diff(x);
	ym = (y(2:end,:)+y(1:end-1,:))/2;
	I = squeeze(sum(bsxfun(@times,ym,reshape(dx,length(dx),1)), 1));
end


% similar to integrateVector(..), but omitting NaNs in y 
function I = integrateVector_nansafe(x, y, average)
	NC = size(y,2); % number of components
	I  = zeros(NC,1);
	for c=1:NC
		valids = find(~isnan(y(:,c)));
		xv = x(valids);
		yv = y(valids,c);
		dx = diff(xv);
		ym = (yv(2:end)+yv(1:end-1))/2;
		I(c) = squeeze(sum(bsxfun(@times,ym,reshape(dx,length(dx),1)), 1));
		if(average)
			I(c) = I(c)/(xv(end)-xv(1));
		end
	end
end




function R2 = getProfileR2(model_z, model_values, data_z, data_values)
	% only keep data within the modeled domain
	valids 		= find((data_z>=model_z(1)) & (data_z<=model_z(end)));
	data_z 		= data_z(valids);
	data_values = data_values(valids);
	
	% interpolate model values at data z
	model_values_at_data_z = interp1q(model_z, model_values, data_z);
	valids = not(isnan(model_values_at_data_z)); 	% some depths might return NaN in interpolation, e.g. when outside of interpolation domain. 
	if(not(any(valids)))
		% no valid model or data values available at any data depth
		R2 = NaN; 
		return; 
	end
	count	= sum(valids);
	
	SSR		= sum((data_values(valids) - model_values_at_data_z(valids)).^2);
	SST		= sum(data_values(valids).^2) - (sum(data_values(valids)))^2/count;
	R2 		= 1 - SSR/SST;
end


% create a parent directory, if it does not exit
function make_parent(path)
	[parent,~,~] = fileparts(path);
	if(not(exist(parent, 'dir')))
		mkdir(parent);
	end
end


function s = sanitize_string_for_latex(s)
	s = strrep(s, '_', '.');
	s = strrep(s, '#', '\#');
	s = strrep(s, '&', '\&');
end


