% LICENSE AGREEMENT
% - - - - - - - - -
% Copyright (c) 2018, Stilianos Louca
% All rights reserved.
%
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are
% met:
%
%    * Redistributions of source code must retain the above copyright
%      notice, this list of conditions and the following disclaimer.
%    * Redistributions in binary form must reproduce the above copyright
%      notice, this list of conditions and the following disclaimer in
%      the documentation and/or other materials provided with the distribution
%    * Neither the name of the original author (Stilianos Louca), nor the names
%      of any other contributors may be used to endorse or promote products derived
%      from this software without specific prior written permission.
%
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
% AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
% IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
% ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
% LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
% CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
% SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
% CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
% ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
% POSSIBILITY OF SUCH DAMAGE.
%
% - - - - - - - - -
% Tested on MATLAB R2015b.
% - - - - - - - - -
%
% DESCRIPTION
% - - - - - -
% Plot scatterplots for N observable pairs
% scatterX(:,j) lists the X-scattervalues of the j-th scatterplot. This can also be a 1D vector, if X values are the same for all scatterplots. It can also be a cell array, listing arbitrary (X,Y) pairs (as an NPj x 2 matrix) for each plot.
% scatterY(:,j) lists the Y-scattervalues of the j-th scatterplot. This can also be a 1D vector, if Y values are the same for all scatterplots.
% titles{} is either a cell array of N strings or a single string. Similarly for xlabels{} and ylabels{}.
function plotScatter(	N, ...					% number of scatterplots
						scatterX,...			% x-coordinates of scatterpoints. Matrix of size NP x N, or a 1D vector of size NP, where NP is the number of scatterpoints. Can also be a cell array of size N, the j-th entry of which is a matrix of size NPj x 2, listing X coordinates in the 1st column and Y coordinates in the 2nd column; in that case, scatterY is ignored.
						scatterY, ...			% y-coordinates of scatterpoints. Matrix of size NP x N, or a 1D vector of size NP, where NP is the number of scatterpoints
						only_points,...			% optional 1D array of integers (ranging from 1 to N), listing scatterpoint indices to be plotted. If empty ([]), all scatterpoints are plotted. In any case, all scatterploints are saved to dataFile.
						rescaleX, ... 			% either NaN or a vector of size N, giving rescaling factors X values in each scatterplot
						rescaleY, ... 			% either NaN or a vector of size N, giving rescaling factors Y values in each scatterplot
						titles, ...
						xrange, ...				% either an empty vector [] or a tuple specifying min & max x value to show
						yrange, ...				% either an empty vector [] or a tuple specifying min & max y value to show
						xlabels, ...
						ylabels, ...
						plotFile,...
						supertitle, ...
						xlogarithmic, ...		% single boolean or an array of booleans of size N, indicating whether to plot logarithmic x axis
						ylogarithmic, ...		% single boolean or an array of booleans of size N, indicating whether to plot logarithmic y axis
						xreverse, ...			% single boolean or an array of booleans of size N, indicating whether to plot he x axis in reverse order
						yreverse, ...			% single boolean or an array of booleans of size N, indicating whether to plot he y axis in reverse order
						plotWidth, ...
						plotHeight, ...
						dataFile) 				% can be '' (empty)	
	% geometry for sub-plot distribution
	[nrows ncols] = num2RowsColumns(N, 0.6,0.2,3);
	titleAreaV	= 0.05 * (1+length(strfind(supertitle,sprintf('\n')))); % reserve title space
	plotAreaV 	= 1-titleAreaV;
	
	% initialize figure window (make invisible since we plot everything to pdf)
	fig = figure('Visible','off','PaperPosition',[0 0 ncols*plotWidth nrows*plotHeight],'PaperSize',[ncols*plotWidth nrows*plotHeight]);

	% iterate through sub plots
	for row=1:nrows
		for col=1:ncols
			j = (row-1)*ncols+col;
			if(j>N) break; end
			
			% determine values for this particular scatterplot
			if(iscell(scatterX))
				if(size(scatterX{j},2)>1)
					X = squeeze(scatterX{j}(:,1));
					Y = squeeze(scatterX{j}(:,2));
				else
					X = [];
					Y = [];
				end
			else
				if(size(scatterX,2)>1)
					X = squeeze(scatterX(:,j));
				else
					X = scatterX;
				end
				if(size(scatterY,2)>1)
					Y = squeeze(scatterY(:,j));
				else
					Y = scatterY;
				end
			end
			if(length(only_points)>0)
				X = X(only_points);
				Y = Y(only_points);
			end
			
			% omit NaNs
			valids = find(not(isnan(X)) & not(isnan(Y)));
			X = X(valids);
			Y = Y(valids);
			
			% rescale values if necessary
			if(not(isnan(rescaleX)))
				X = X .* rescaleX(j);
			end
			if(not(isnan(rescaleY)))
				Y = Y .* rescaleY(j);
			end
			
			% adjust logarithmic choices, e.g. don't use logarithmic axes when all data are zero	
			if(length(xlogarithmic)>1)
				this_xlogarithmic = xlogarithmic(j);
			else
				this_xlogarithmic = xlogarithmic;
			end
			if(this_xlogarithmic && any(X<=0))
				this_xlogarithmic = false;
			end
			if(length(ylogarithmic)>1)
				this_ylogarithmic = ylogarithmic(j);
			else
				this_ylogarithmic = ylogarithmic;
			end
			if(this_ylogarithmic && any(Y<=0))
				this_ylogarithmic = false;
			end
			
			% adjust axis orientation choices
			if(length(xreverse)>1)
				this_xreverse = xreverse(j);
			else
				this_xreverse = xreverse;
			end
			if(length(yreverse)>1)
				this_yreverse = yreverse(j);
			else
				this_yreverse = yreverse;
			end
			
			% mannually place subplot
			% Note: If subplot overlaps previous subplots, the older ones will be deleted, unless overlap is exact! (this is an idiotic matlab 'feature')
			subplot_position = [(0.15+col-1)/ncols plotAreaV*(0.25+nrows-row)/nrows 0.7/ncols plotAreaV*0.6/nrows];
			subplot('Position',subplot_position);
			
			% plot scatterpoints
			scatter(X,Y,1,'MarkerFaceAlpha',0.5,'MarkerEdgeAlpha',0.6);
			
			% set title & axis labels
			if(iscell(titles)) thisTitle = titles{j}; else thisTitle = titles; end
			title((thisTitle));
			if(iscell(xlabels))
				xlabel(sanitize_string_for_latex(xlabels{j}),'Interpreter','latex')
			else
				xlabel(sanitize_string_for_latex(xlabels),'Interpreter','latex')
			end
			if(iscell(ylabels))
				ylabel(sanitize_string_for_latex(ylabels{j}),'Interpreter','latex')
			else
				ylabel(sanitize_string_for_latex(ylabels),'Interpreter','latex')
			end
			set(gca,'fontsize',8);
			
			if(length(X)==0)
				% no data to plot
				title(sprintf('%s\nNo data available',thisTitle));
				continue;
			end

			% set x-axis limits
			[minx, maxx] = get_range(X, xrange, this_xlogarithmic);
			if(isnan(minx) | isnan(maxx))
				continue; % something is wrong with the data, so skip
			end
			xlim([minx maxx]);
			if(this_xlogarithmic) set(gca, 'XScale', 'log'); end
			if(this_xreverse) set(gca,'XDir','reverse'); end

			% set y-axis limits
			[miny, maxy] = get_range(Y, yrange, this_ylogarithmic);
			if(isnan(miny) | isnan(maxy))
				continue; % something is wrong with the data, so skip
			end
			ylim([miny maxy]);
			if(this_ylogarithmic) set(gca, 'YScale', 'log'); end
			if(this_yreverse) set(gca,'YDir','reverse'); end
									
		end
	end
		
	%create super title
	%this should come after subplot(), because the latter has a bad tendency to delete previous figure elements
	annotation('textbox', [0 (1-titleAreaV) 1 titleAreaV], 'String', sanitize_string_for_latex(supertitle), 'EdgeColor', 'none', 'HorizontalAlignment', 'center', 'VerticalAlignment', 'top','Interpreter','latex');

	make_parent(plotFile);
	print(fig,'-dpdf',plotFile);
	
	% Write data to file
	if(not(isempty(dataFile)))
		index = 0;
		make_parent(dataFile);
		fout = fopen(dataFile, 'w');
		fprintf(fout, '# %s\n#\n', strrep(supertitle, sprintf('\n'), sprintf('\n#   ')));
		for j=1:N
			% write scatterdata
			X = []; Y = [];
			if(iscell(scatterX))
				if(size(scatterX{j},2)>1)
					X = scatterX{j}(:,1);
					Y = scatterX{j}(:,2);
				end
			else
				if(size(scatterX,2)>1) X = scatterX(:,j); else X = scatterX; end
				if(size(scatterY,2)>1) Y = scatterY(:,j); else Y = scatterY; end
			end
			if(iscell(titles)) tt = titles{j}; else tt = titles; end
			if(iscell(xlabels)) txl=xlabels{j}; else txl = xlabels; end
			if(iscell(ylabels)) tyl=ylabels{j}; else tyl = ylabels; end
			fprintf(fout,'# Data index %d\n# Time series: %s\n# Data columns: %s\t%s\n', index, tt, txl, tyl);
			for p=1:length(X)
				fprintf(fout, '%g\t%g\n', X(p), Y(p)); 
			end
			fprintf(fout, '\n\n'); index = index + 1;
		end
		fclose(fout);
	end
end



function [min_value,max_value] = get_range(values, explicit_range, logarithmic)
	if(length(explicit_range)>1)
		min_value = explicit_range(1);
		max_value = explicit_range(2);
	else
		% remove invalid values
		if(logarithmic)
			values = values((~isnan(values)) & (~isinf(values)) & (values>0));
		else
			values = values((~isnan(values)) & (~isinf(values)));
		end
		if(length(values)==0)
			min_value = NaN;
			max_value = NaN;
		elseif(logarithmic)
			values 		= log(values);
			min_value 	= min(values);
			max_value 	= max(values);
			L 			= (max_value-min_value);
			min_value 	= min_value-0.1*L;
			max_value 	= max_value+0.1*L;
			min_value	= exp(min_value);
			max_value	= exp(max_value);
			if(min_value==max_value)
				min_value = 0.99*min_value;
				max_value = 1.01*max_value;
			end
		else
			min_value 	= min(values);
			max_value 	= max(values);
			L 			= (max_value-min_value);
			min_value 	= ternary(min_value<0, min_value-0.1*L, max(0,min_value-0.1*L));
			max_value 	= ternary(max_value>0, max_value+0.1*L, min(0,max_value+0.1*L));
			if(min_value==max_value)
				if(min_value==0)
					max_value = 1;
				else
					min_value = min_value-0.01*abs(min_value);
					max_value = max_value+0.01*abs(max_value);
				end
			end
		end
	end
end



% Ternary operator for C-like conditional expressions
function c = ternary(condition, a , b)
    if condition
        c = a;
    else
        c = b;
    end
end


% create a parent directory, if it does not exit
function make_parent(path)
	[parent,~,~] = fileparts(path);
	if(not(exist(parent, 'dir')))
		mkdir(parent);
	end
end

function s = sanitize_string_for_latex(s)
	s = strrep(s, '_', '.');
	s = strrep(s, '#', '\#');
end
